class EnvelopePerimeter:
    """
    A spatial envelope defined by the bounding coordinates. The spatial reference must match the 
    spatial reference of the datasets.
    """
    def __init__(self, x_min: float, y_min: float, x_max: float,  y_max: float):
        if x_min is None or x_max is None or y_min is None or y_max is None:
            raise Exception('Not all parameters are defined. Please define all envelope parameters')
        self.x_min = x_min
        self.x_max = x_max
        self.y_min = y_min
        self.y_max = y_max


class EsriShapePerimeter:
    """
    A polygon in the Esri shape buffer format.
    """
    def __init__(self, esri_shape: bytes):
        self.esri_shape = esri_shape


class WkbPerimeter:
    """
    A polygon in the OGC well-known-binary format. For example, in ArcPy the geometry's WKB 
    property could be used to acquire this format.
    """
    def __init__(self, wkb: bytes):
        self.wkb = wkb


import typing


class VerificationParameters():
    """
    Contains all parameters that can be passed to a verification. 

    :param tile_size: size (in meter) for testing quality conditions
    :type tile_size: int
    :param username: the executing user
    :type username: str
    """

    def __init__(self, tile_size: int = 5000, user_name: str = None) -> None:
        #:
        self.tile_size: int = tile_size
        #:
        self.user_name: str = user_name



from prosuite.dataset import Dataset


class Parameter:
    """
    A parameter configures a quality condition. Parameters can represent Datasets (dataset parameter) or scalar values
    (scalar parameters). Parameters have a name and a value.

    Dataset parameters: value is of type dataset. the parameter can retrieve the workspace id (model name) and the where
    clause (filter expression) of the dataset.

    Scalar parameters: value is a simple type (number, string, bool). .
    """
    def __init__(self, name: str, value):
        self.contains_list_of_datasets = False
        if self._value_type_is_dataset(value):
            self.dataset = value
        else:
            self.dataset = None

        self.name: str = name
        self.value = value

    def get_string_value(self) -> str:
        if self.dataset:
            if self.contains_list_of_datasets:
                return self.dataset[0].name
            else:
                return self.dataset.name
        else:
            if self.value is None:
                return ""
            else:
                return str(self.value)

    def is_dataset_parameter(self) -> bool:
        if self.dataset:
            return True
        else:
            return False

    def get_workspace_id(self):
        if not self.dataset:
            return ""
        return self.dataset.model.name

    def get_where_clause(self):
        if not self.dataset:
            return ""
        if self.contains_list_of_datasets:
            return self.dataset[0].filter_expression
        else:
            return self.dataset.filter_expression

    @staticmethod
    def value_is_list_of_datasets(value):
        if isinstance(value, list):
            if all(isinstance(x, Dataset) for x in value):
                return True
        return False

    def _value_type_is_dataset(self, value):
        if Parameter.value_is_list_of_datasets(value):
            self.contains_list_of_datasets = True
            return True
        if isinstance(value, Dataset):
            return True
        return False



import typing
from typing import List
import prosuite.utils as utils


class XmlSpecification:
    """
    Represents a specification defined in the xml specification schema:

    :param specification_file: path to the xml specification file
    :type specification_file: str
    :param specification_name: name of the specification (in the xml file) that should be executed. If not defined in the constructor, it needs to be defined later.
    :type specification_name: str
    :param data_source_replacements: a list containing a list with two string elements. these represent a workspace id, and the path to the workspace
    :type data_source_replacements: [[str]] example: [["TLM_Production", "C:/temp/user@topgist.sde"]]
    """

    def __init__(self, specification_file: str, specification_name: str = None,
                 data_source_replacements: List[List[str]] = None):

        self.specification_name: str = None
        self.data_source_replacements = XmlSpecification._parse_datasource_replacements(data_source_replacements)
        self._specification_msg = None
        self.xml_string = XmlSpecification._read_file_as_string(specification_file)
        self.specification_file = specification_file
        self.specification_name = specification_name

    @staticmethod
    def _read_file_as_string(file_path) -> str:
        with open(file_path, "r", encoding="utf-8") as file:
            data = file.read()
            return data

    @staticmethod
    def _parse_datasource_replacements(data_source_replacements: List[List[str]]):
        result = []
        for pair in data_source_replacements:
            result.append(f"{pair[0]}|{pair[1]}")
        return result

    @staticmethod
    def get_specification_names(specification_file: str) -> typing.List[str]:
        """
        Returns a list of the specification names of all specifications in all categories of the xml-specifications doc.

        :param specification_file: path of xml specification file.
        :type specification_file: str
        :return: List of specification names
        :rtype: [str]
        """
        specification_names = []
        xml = utils.objectify_xml(specification_file)
        for cat in xml.Categories:
            for qs in cat.Category.QualitySpecifications:
                name = utils.try_get_lxml_attrib(qs.QualitySpecification, 'name')
                if name:
                    specification_names.append(name)
        return specification_names



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import quality_verification_service_pb2 as quality__verification__service__pb2


class QualityVerificationGrpcStub(object):
    """*
    The quality verification service definition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.VerifyQuality = channel.unary_stream(
                '/ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc/VerifyQuality',
                request_serializer=quality__verification__service__pb2.VerificationRequest.SerializeToString,
                response_deserializer=quality__verification__service__pb2.VerificationResponse.FromString,
                )
        self.VerifyDataQuality = channel.stream_stream(
                '/ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc/VerifyDataQuality',
                request_serializer=quality__verification__service__pb2.DataVerificationRequest.SerializeToString,
                response_deserializer=quality__verification__service__pb2.DataVerificationResponse.FromString,
                )
        self.VerifyStandaloneXml = channel.unary_stream(
                '/ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc/VerifyStandaloneXml',
                request_serializer=quality__verification__service__pb2.StandaloneVerificationRequest.SerializeToString,
                response_deserializer=quality__verification__service__pb2.StandaloneVerificationResponse.FromString,
                )


class QualityVerificationGrpcServicer(object):
    """*
    The quality verification service definition.
    """

    def VerifyQuality(self, request, context):
        """*
        Verifies the quality definied in the data dictionary.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifyDataQuality(self, request_iterator, context):
        """*
        Experimental - Verifies the data quality definied in the data dictionary. The data is provided by the client.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifyStandaloneXml(self, request, context):
        """*
        Verifies the quality using an XML definition instead of accessing the data dictionary.
        This is a simplified interface with a basic response and no parallelism. Consider using
        VerifyQuality instead.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QualityVerificationGrpcServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'VerifyQuality': grpc.unary_stream_rpc_method_handler(
                    servicer.VerifyQuality,
                    request_deserializer=quality__verification__service__pb2.VerificationRequest.FromString,
                    response_serializer=quality__verification__service__pb2.VerificationResponse.SerializeToString,
            ),
            'VerifyDataQuality': grpc.stream_stream_rpc_method_handler(
                    servicer.VerifyDataQuality,
                    request_deserializer=quality__verification__service__pb2.DataVerificationRequest.FromString,
                    response_serializer=quality__verification__service__pb2.DataVerificationResponse.SerializeToString,
            ),
            'VerifyStandaloneXml': grpc.unary_stream_rpc_method_handler(
                    servicer.VerifyStandaloneXml,
                    request_deserializer=quality__verification__service__pb2.StandaloneVerificationRequest.FromString,
                    response_serializer=quality__verification__service__pb2.StandaloneVerificationResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class QualityVerificationGrpc(object):
    """*
    The quality verification service definition.
    """

    @staticmethod
    def VerifyQuality(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc/VerifyQuality',
            quality__verification__service__pb2.VerificationRequest.SerializeToString,
            quality__verification__service__pb2.VerificationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifyDataQuality(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc/VerifyDataQuality',
            quality__verification__service__pb2.DataVerificationRequest.SerializeToString,
            quality__verification__service__pb2.DataVerificationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifyStandaloneXml(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/ProSuite.Microservices.Definitions.QA.QualityVerificationGrpc/VerifyStandaloneXml',
            quality__verification__service__pb2.StandaloneVerificationRequest.SerializeToString,
            quality__verification__service__pb2.StandaloneVerificationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import os
import sys
# we need to add the current path (of the package 'generated') to the python paths so that the
# files generated with protoc can be imported.
# reason: protoc does not write fully namespaced import statements
# (no option to add a package name to the import statement). Therefore
# when prosuite is installed, the generated grpc files are not referenced correctly.
sys.path.append(os.path.dirname(os.path.abspath(__file__)))



from prosuite.condition import Condition
import typing


class Specification:
    def __init__(self, name: str = 'Custom Specification', description: str = ''):
        """
        CustomSpecification stores conditions.
        :param name: specification name
        :type name: str
        :param description: specification description
        :type description: str
        """
        self._conditions: typing.List[Condition] = []
        self.name = name
        self.description = description

    def add_condition(self, condition: Condition):
        """
        Adds conditions to the specification
        """
        self._conditions.append(condition)

    def get_conditions(self) -> typing.List[Condition]:
        """
        Returns the List of conditions
        """
        return self._conditions



__author__ = "The ProSuite Authors"
__copyright__ = "Copyright 2021-2023, The ProSuite Authors"
__license__ = "MIT"
__version__ = "1.0.1.0"
__maintainer__ = "Dira GeoSystems"
__email__ = "programmers@dirageosystems.ch"
__date__  = "16.01.2023"
__status__ = "Production"


from datetime import datetime
from typing import List
from prosuite.transformer import Transformer
from prosuite.parameter import Parameter
from prosuite.dataset import BaseDataset
from prosuite.factories.enums import *

class Transformers:
    @classmethod
    def not_yet_implemented(cls, feature_class: BaseDataset, tolerance: float) -> Transformer:
        """
        Stay tuned
        """
        
        result = Transformer("dummy(0)")
        return result




class Model:
    """
    The Model represents the data model in a workspace (file-gdb or enterprise-gdb)

    catalog_path examples:
        c:/data.gdb
        c:/enterprise_gdb.sde
    """

    def __init__(self, name, catalog_path):

        #:
        self.name: str = name
        """
        The unique name of the model.
        """
        #:
        self.catalog_path: str = catalog_path
        """
        The catalog path of the associated workspace.
        """



__all__ = ['Condition']
from prosuite.issue_filter import IssueFilter
from prosuite.parameter import Parameter
from typing import List


class Condition:
    """
    Defines a quality condition, i.e. the configuration of a test algorithm with for one or more 
    datasets. Conditions must be created with the factory methods from :py:mod:`prosuite.factories.quality_conditions` 
    """

    def __init__(self, test_descriptor: str, name: str = ""):

        #:
        self.name = name
        """
        The unique name of the quality condition.
        """
        #:
        self.test_descriptor = test_descriptor
        """
        The test descriptor, i.e. the algorithm used to verify this condition.
        """
        
        #: TODO: Rename to IssueType or add enum IssueType {Warning|Error} or Serverity 
        # New doc: The type/severity of the quality issues identified by this quality condition. Quality conditions with 
        # “Issue Type = Warning” are considered “soft” conditions, for which exceptions (“Allowed Issues”) may be defined.
        self.allow_errors: bool = False
        """
        For internal use only.
        """
        #:
        self.category: str = ""
        """
        The name of the category, if this issue is assigned to a category.
        """
        #:
        self.stop_on_error: bool = False
        """
        Indicates if the occurrence of an error for an object should stop any further testing of the same object. 
        This can be used to prevent further tests on a feature after a serious geometry error (e.g. incorrectly 
        oriented rings) was detected for the feature.
        The used Test Descriptor provides a standard value for this property. It can optionally be overridden here.
        """
        #:
        self.description: str = ""
        """
        Freely definable description of the Quality Condition. This description can be displayed when viewing 
        issues in the issue navigator, and may contain explanations to the Quality Condition and instructions for 
        correcting the issues.
        """
        #:
        self.url: str = ""
        """
        Optional URL to a website providing additional information for this Quality Condition.
        Certain Quality Conditions require more detailed information about the test logic and/or the correction 
        guidelines than the field “Description” can provide. This information can for example be assembled in a 
        wiki, and the URL may be provided here. When viewing issues in the issue navigator, the corresponding web 
        page can be opened. In the HTML verification reports these URLs are used to render the names of the 
        Quality Conditions as links.
        """
        #:
        self.parameters: List[Parameter] = []
        """
        The list of parameters. Typically the parameters are specified in the factory method used to create the
        quality condition (see :py:mod:`prosuite.factories.quality_conditions`) and do not need to be changed
        through this list.
        """
        #:
        self.issue_filters: List[IssueFilter] = []
        """
        Reserved for future use.
        """
        #:
        self.issue_filter_expression: str
        """
        Reserved for future use.
        """

    def generate_name(self):
        """
        Generates a technical name using the dataset name(s) and the test descriptor. This is the default name of 
        a condition if it was created by the standard factory method from :py:mod:`prosuite.factories.quality_conditions`.
        """
        first_dataset_parameter = next(
            (p for p in self.parameters if p.is_dataset_parameter), None)
        if first_dataset_parameter:
            ds_param: Parameter = first_dataset_parameter
            if ds_param.contains_list_of_datasets:
                dataset_list: List[str] = [ds.name for ds in ds_param.dataset]
                dataset_names = "_".join(dataset_list)
                self.name = f"{self.test_descriptor} {dataset_names}"
            else:
                self.name = f"{self.test_descriptor} {ds_param.dataset.name}"



class VerificationResponse:
    """
    This class represents a VerificationResponse Message.

    :param message: the actual message
    :param message_level: message level -> see class MessageLevel
    :param service_call_status: service status -> see class ServiceStatus

    The str() method is overridden to return all properties from the VerificationResponse when 
    a VerificationResponse object is printed using pythons print() method. 

    """

    def __init__(self, service_call_status: str, message: str, message_level: str):
        #:
        self.message: str = message
        #:
        self.service_call_status = ServiceStatus.status_0
        #:
        self.message_level = MessageLevel.level_40000
        self._set_service_call_status(service_call_status)
        self._set_message_level(message_level)

    def __str__(self):
        return f'service_call_status: {self.service_call_status}\t message_level: {self.message_level} \t ' \
               f'message: {self.message}'

    def _set_service_call_status(self, service_call_status: str):
        if service_call_status == 0:
            self.service_call_status = ServiceStatus.status_0
        if service_call_status == 1:
            self.service_call_status = ServiceStatus.status_1
        if service_call_status == 2:
            self.service_call_status = ServiceStatus.status_2
        if service_call_status == 3:
            self.service_call_status = ServiceStatus.status_3
        if service_call_status == 4:
            self.service_call_status = ServiceStatus.status_4

    def _set_message_level(self, message_level: str):
        if message_level == 110000:
            self.message_level = MessageLevel.level_110000
        if message_level == 70000:
            self.message_level = MessageLevel.level_70000
        if message_level == 60000:
            self.message_level = MessageLevel.level_60000
        if message_level == 40000:
            self.message_level = MessageLevel.level_40000
        if message_level == 30000:
            self.message_level = MessageLevel.level_30000
        if message_level == 10000:
            self.message_level = MessageLevel.level_10000


class ServiceStatus:
    status_0 = 'Undefined'
    status_1 = 'Running'
    status_2 = 'Cancelled'
    status_3 = 'Finished'
    status_4 = 'Failed'


class MessageLevel:
    level_110000 = 'Fatal'
    level_70000 = 'Error'
    level_60000 = 'Warn'
    level_40000 = "Info"
    level_30000 = 'Debug'
    level_10000 = 'Verbose'



from typing import List
from prosuite.parameter import Parameter


class IssueFilter():
    def __init__(self, name, expression: str):
        self.parameters: List[Parameter] = []
        self.name = name
        self.expression = expression



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: quality_test.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import shared_types_pb2 as shared__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12quality_test.proto\x12*ProSuite.Microservices.Definitions.QA.Test\x1a\x12shared_types.proto\"\xa5\x02\n\x12\x45xecuteTestRequest\x12K\n\nworkspaces\x18\x01 \x03(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.WorkspaceMsg\x12S\n\x0finvolved_tables\x18\x02 \x03(\x0b\x32:.ProSuite.Microservices.Definitions.QA.Test.TestDatasetMsg\x12\x46\n\tperimeter\x18\x03 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\x12\x11\n\ttest_name\x18\x04 \x01(\t\x12\x12\n\nparameters\x18\x05 \x03(\t\"\xcf\x01\n\x13\x45xecuteTestResponse\x12\x1b\n\x13service_call_status\x18\x01 \x01(\x05\x12M\n\x08progress\x18\x02 \x01(\x0b\x32;.ProSuite.Microservices.Definitions.QA.Test.TestProgressMsg\x12L\n\x06issues\x18\x03 \x03(\x0b\x32<.ProSuite.Microservices.Definitions.QA.Test.DetectedIssueMsg\"\x80\x01\n\x0eTestDatasetMsg\x12S\n\x10\x63lass_definition\x18\x01 \x01(\x0b\x32\x39.ProSuite.Microservices.Definitions.Shared.ObjectClassMsg\x12\x19\n\x11\x66ilter_expression\x18\x02 \x01(\t\"_\n\x0fTestProgressMsg\x12\x1c\n\x14progress_total_steps\x18\x01 \x01(\x05\x12\x1d\n\x15progress_current_step\x18\x02 \x01(\x05\x12\x0f\n\x07message\x18\x03 \x01(\t\"\xdb\x02\n\x10\x44\x65tectedIssueMsg\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\x12K\n\x0eissue_geometry\x18\x02 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\x12\x15\n\rissue_code_id\x18\x03 \x01(\t\x12\x1e\n\x16issue_code_description\x18\x04 \x01(\t\x12X\n\x10involved_objects\x18\x05 \x03(\x0b\x32>.ProSuite.Microservices.Definitions.QA.Test.InvolvedObjectsMsg\x12\x16\n\x0estop_condition\x18\x06 \x01(\x08\x12\x1a\n\x12\x61\x66\x66\x65\x63ted_component\x18\x07 \x01(\t\x12 \n\x18\x63reation_date_time_ticks\x18\x08 \x01(\x03\"t\n\x12InvolvedObjectsMsg\x12J\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x39.ProSuite.Microservices.Definitions.Shared.ObjectClassMsg\x12\x12\n\nobject_ids\x18\x02 \x03(\x03\x32\xa2\x01\n\x0fQualityTestGrpc\x12\x8e\x01\n\x07\x45xecute\x12>.ProSuite.Microservices.Definitions.QA.Test.ExecuteTestRequest\x1a?.ProSuite.Microservices.Definitions.QA.Test.ExecuteTestResponse\"\x00\x30\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'quality_test_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EXECUTETESTREQUEST._serialized_start=87
  _EXECUTETESTREQUEST._serialized_end=380
  _EXECUTETESTRESPONSE._serialized_start=383
  _EXECUTETESTRESPONSE._serialized_end=590
  _TESTDATASETMSG._serialized_start=593
  _TESTDATASETMSG._serialized_end=721
  _TESTPROGRESSMSG._serialized_start=723
  _TESTPROGRESSMSG._serialized_end=818
  _DETECTEDISSUEMSG._serialized_start=821
  _DETECTEDISSUEMSG._serialized_end=1168
  _INVOLVEDOBJECTSMSG._serialized_start=1170
  _INVOLVEDOBJECTSMSG._serialized_end=1286
  _QUALITYTESTGRPC._serialized_start=1289
  _QUALITYTESTGRPC._serialized_end=1451
# @@protoc_insertion_point(module_scope)



"""
prosuite is a ProSuite client API. It supports creation, configuration and execution of QA conditions.
"""
# import generated

import prosuite.factories.enums as Enums
import prosuite.utils as utils
from prosuite.transformer import Transformer
from prosuite.factories.quality_conditions import Conditions
from prosuite.factories.transformers import Transformers
from prosuite.factories.issue_filters import IssueFilters
from prosuite.dataset import Dataset
from prosuite.transformed_dataset import TransformedDataset
from prosuite.service import Service
from prosuite.condition import Condition
from prosuite.specification import Specification
from prosuite.xml_specification import XmlSpecification
from prosuite.model import Model
from prosuite.parameter import Parameter
from prosuite.perimeter import WkbPerimeter, EnvelopePerimeter, EsriShapePerimeter
from prosuite.verification_parameters import VerificationParameters



from prosuite.dataset import BaseDataset
from prosuite.parameter import Parameter
from prosuite.model import Model
from typing import List


class TransformedDataset(BaseDataset):
    def __init__(self, name: str, filter_expression: str = ""):
        super().__init__(name, filter_expression)

        #:
        self.parameters: List[Parameter] = []



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: shared_qa.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import shared_types_pb2 as shared__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fshared_qa.proto\x12%ProSuite.Microservices.Definitions.QA\x1a\x12shared_types.proto\"\xda\x01\n\x0eWorkContextMsg\x12\x0e\n\x06\x64\x64x_id\x18\x01 \x01(\x05\x12\x0c\n\x04type\x18\x02 \x01(\x05\x12J\n\tworkspace\x18\x03 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.WorkspaceMsg\x12\x14\n\x0cversion_name\x18\x06 \x01(\t\x12\x1c\n\x14verified_dataset_ids\x18\x07 \x03(\x05\x12\x14\n\x0c\x63ontext_type\x18\x04 \x01(\t\x12\x14\n\x0c\x63ontext_name\x18\x05 \x01(\t\"\xe0\x02\n\x17QualitySpecificationMsg\x12\"\n\x18quality_specification_id\x18\x01 \x01(\x05H\x00\x12\"\n\x18well_known_specification\x18\x02 \x01(\x05H\x00\x12^\n\x11xml_specification\x18\x03 \x01(\x0b\x32\x41.ProSuite.Microservices.Definitions.QA.XmlQualitySpecificationMsgH\x00\x12l\n\x1c\x63ondition_list_specification\x18\x05 \x01(\x0b\x32\x44.ProSuite.Microservices.Definitions.QA.ConditionListSpecificationMsgH\x00\x12\x1e\n\x16\x65xcluded_condition_ids\x18\x04 \x03(\x05\x42\x0f\n\rspecification\"p\n\x1aXmlQualitySpecificationMsg\x12\x0b\n\x03xml\x18\x01 \x01(\t\x12#\n\x1bselected_specification_name\x18\x02 \x01(\t\x12 \n\x18\x64\x61ta_source_replacements\x18\x03 \x03(\t\"\xe7\x01\n\x1d\x43onditionListSpecificationMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12W\n\x08\x65lements\x18\x03 \x03(\x0b\x32\x45.ProSuite.Microservices.Definitions.QA.QualitySpecificationElementMsg\x12J\n\x0c\x64\x61ta_sources\x18\x04 \x03(\x0b\x32\x34.ProSuite.Microservices.Definitions.QA.DataSourceMsg\"m\n\rDataSourceMsg\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_path\x18\x02 \x01(\t\x12\x12\n\nmodel_name\x18\x03 \x01(\t\x12\x10\n\x08\x64\x61tabase\x18\x04 \x01(\t\x12\x14\n\x0cschema_owner\x18\x05 \x01(\t\"\xb3\x01\n\x1eQualitySpecificationElementMsg\x12M\n\tcondition\x18\x01 \x01(\x0b\x32:.ProSuite.Microservices.Definitions.QA.QualityConditionMsg\x12\x14\n\x0c\x61llow_errors\x18\x02 \x01(\x08\x12\x15\n\rstop_on_error\x18\x03 \x01(\x08\x12\x15\n\rcategory_name\x18\x05 \x01(\t\"\xc5\x02\n\x13QualityConditionMsg\x12\x14\n\x0c\x63ondition_id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x1c\n\x14test_descriptor_name\x18\x03 \x01(\t\x12\x0b\n\x03url\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12G\n\nparameters\x18\x06 \x03(\x0b\x32\x33.ProSuite.Microservices.Definitions.QA.ParameterMsg\x12`\n\x17\x63ondition_issue_filters\x18\x07 \x03(\x0b\x32?.ProSuite.Microservices.Definitions.QA.InstanceConfigurationMsg\x12\x1f\n\x17issue_filter_expression\x18\x08 \x01(\t\"\xc1\x01\n\x18InstanceConfigurationMsg\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12 \n\x18instance_descriptor_name\x18\x03 \x01(\t\x12\x0b\n\x03url\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12G\n\nparameters\x18\x06 \x03(\x0b\x32\x33.ProSuite.Microservices.Definitions.QA.ParameterMsg\"\xcd\x01\n\x0cParameterMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12T\n\x0btransformer\x18\x06 \x01(\x0b\x32?.ProSuite.Microservices.Definitions.QA.InstanceConfigurationMsg\x12\x14\n\x0cwhere_clause\x18\x03 \x01(\t\x12\x14\n\x0cworkspace_id\x18\x04 \x01(\t\x12\x1e\n\x16used_as_reference_data\x18\x05 \x01(\x08\"\xbe\x01\n\x15InstanceDescriptorMsg\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x05\x12S\n\x10\x63lass_descriptor\x18\x04 \x01(\x0b\x32\x39.ProSuite.Microservices.Definitions.QA.ClassDescriptorMsg\x12\x13\n\x0b\x63onstructor\x18\x05 \x01(\x05\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\">\n\x12\x43lassDescriptorMsg\x12\x11\n\ttype_name\x18\x01 \x01(\t\x12\x15\n\rassembly_name\x18\x02 \x01(\t\"\xda\x05\n\x19VerificationParametersMsg\x12\x11\n\ttile_size\x18\x01 \x01(\x01\x12\x46\n\tperimeter\x18\x02 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\x12*\n\"write_detailed_verification_report\x18\x03 \x01(\x08\x12 \n\x18verification_report_path\x18\x04 \x01(\t\x12\x18\n\x10html_report_path\x18\x05 \x01(\t\x12\x1a\n\x12html_template_path\x18\x06 \x01(\t\x12\x1b\n\x13issue_file_gdb_path\x18\x07 \x01(\t\x12\x33\n+store_related_geometry_for_table_row_issues\x18\x08 \x01(\x08\x12\x30\n(filter_table_rows_using_related_geometry\x18\t \x01(\x08\x12\x1f\n\x17override_allowed_errors\x18\n \x01(\x08\x12\'\n\x1freport_issues_outside_perimeter\x18\x0b \x01(\x08\x12/\n\'force_full_scan_for_non_container_tests\x18\x0c \x01(\x08\x12$\n\x1csave_verification_statistics\x18\r \x01(\x08\x12 \n\x18report_unused_exceptions\x18\x0e \x01(\x08\x12!\n\x19report_invalid_exceptions\x18\x0f \x01(\x08\x12<\n4invalidate_exceptions_if_any_involved_object_changed\x18\x10 \x01(\x08\x12\x36\n.invalidate_exceptions_if_condition_was_updated\x18\x11 \x01(\x08\"q\n\x07GdbData\x12\x18\n\x10gdb_object_count\x18\x04 \x01(\x03\x12L\n\x0bgdb_objects\x18\x03 \x03(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjectMsg\"\xc0\x02\n\x0b\x44\x61taRequest\x12\x44\n\tclass_def\x18\x01 \x01(\x0b\x32/.ProSuite.Microservices.Definitions.QA.ClassDefH\x00\x12V\n\rrel_query_def\x18\x02 \x01(\x0b\x32=.ProSuite.Microservices.Definitions.QA.RelationshipClassQueryH\x00\x12\x14\n\x0cwhere_clause\x18\x03 \x01(\t\x12L\n\x0fsearch_geometry\x18\x04 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\x12\x12\n\nsub_fields\x18\x05 \x01(\t\x12\x12\n\ncount_only\x18\x06 \x01(\x08\x42\x07\n\x05table\":\n\x08\x43lassDef\x12\x14\n\x0c\x63lass_handle\x18\x01 \x01(\x03\x12\x18\n\x10workspace_handle\x18\x02 \x01(\x03\"\x8c\x01\n\x16RelationshipClassQuery\x12\x1f\n\x17relationship_class_name\x18\x01 \x01(\t\x12\x18\n\x10workspace_handle\x18\x02 \x01(\x03\x12\x0e\n\x06tables\x18\x03 \x03(\t\x12\x11\n\tjoin_type\x18\x04 \x01(\x05\x12\x14\n\x0cwhere_clause\x18\x05 \x01(\t\"\xba\x01\n\tSchemaMsg\x12T\n\x11\x63lass_definitions\x18\x01 \x03(\x0b\x32\x39.ProSuite.Microservices.Definitions.Shared.ObjectClassMsg\x12W\n\x14relclass_definitions\x18\x02 \x03(\x0b\x32\x39.ProSuite.Microservices.Definitions.Shared.ObjectClassMsg\"\x87\x01\n\rSchemaRequest\x12\x13\n\x0b\x64\x61taset_ids\x18\x01 \x03(\x05\x12\x61\n\x1arelationship_class_queries\x18\x02 \x03(\x0b\x32=.ProSuite.Microservices.Definitions.QA.RelationshipClassQuery\"\xc5\x03\n\x17VerificationProgressMsg\x12\x15\n\rprogress_type\x18\x01 \x01(\x05\x12\x15\n\rprogress_step\x18\x02 \x01(\x05\x12$\n\x1coverall_progress_total_steps\x18\x03 \x01(\x05\x12%\n\x1doverall_progress_current_step\x18\x04 \x01(\x05\x12%\n\x1d\x64\x65tailed_progress_total_steps\x18\x05 \x01(\x05\x12\'\n\x1f\x64\x65tailed_progress__current_step\x18\x06 \x01(\x05\x12\x1f\n\x17processing_step_message\x18\x07 \x01(\t\x12K\n\x0b\x63urrent_box\x18\x08 \x01(\x0b\x32\x36.ProSuite.Microservices.Definitions.Shared.EnvelopeMsg\x12I\n\ttotal_box\x18\t \x01(\x0b\x32\x36.ProSuite.Microservices.Definitions.Shared.EnvelopeMsg\x12\x0f\n\x07message\x18\n \x01(\t\x12\x15\n\rmessage_level\x18\x0b \x01(\x05\"\xa0\x03\n\x08IssueMsg\x12\x14\n\x0c\x63ondition_id\x18\x01 \x01(\x05\x12K\n\x0eissue_geometry\x18\x02 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x15\n\rissue_code_id\x18\x05 \x01(\t\x12\x1e\n\x16issue_code_description\x18\x06 \x01(\t\x12P\n\x0finvolved_tables\x18\x07 \x03(\x0b\x32\x37.ProSuite.Microservices.Definitions.QA.InvolvedTableMsg\x12\x1c\n\x14legacy_involved_rows\x18\x08 \x01(\t\x12\x11\n\tallowable\x18\t \x01(\x08\x12\x16\n\x0estop_condition\x18\n \x01(\x08\x12\x1a\n\x12\x61\x66\x66\x65\x63ted_component\x18\x0b \x01(\t\x12 \n\x18\x63reation_date_time_ticks\x18\x0c \x01(\x03\x12\x0c\n\x04test\x18\r \x01(\x05\":\n\x10InvolvedTableMsg\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x12\n\nobject_ids\x18\x02 \x03(\x03\"\xb8\x04\n\x16QualityVerificationMsg\x12\x1d\n\x15saved_verification_id\x18\x01 \x01(\x05\x12\x18\n\x10specification_id\x18\x02 \x01(\x05\x12\x1a\n\x12specification_name\x18\x03 \x01(\t\x12!\n\x19specification_description\x18\x04 \x01(\t\x12\x11\n\tuser_name\x18\x05 \x01(\t\x12\x18\n\x10start_time_ticks\x18\x06 \x01(\x03\x12\x16\n\x0e\x65nd_time_ticks\x18\x07 \x01(\x03\x12\x11\n\tfulfilled\x18\x08 \x01(\x08\x12\x11\n\tcancelled\x18\t \x01(\x08\x12\x1e\n\x16processor_time_seconds\x18\n \x01(\x01\x12\x14\n\x0c\x63ontext_type\x18\x0b \x01(\t\x12\x14\n\x0c\x63ontext_name\x18\x0c \x01(\t\x12!\n\x19rows_with_stop_conditions\x18\r \x01(\x05\x12g\n\x17\x63ondition_verifications\x18\x0e \x03(\x0b\x32\x46.ProSuite.Microservices.Definitions.QA.QualityConditionVerificationMsg\x12\x63\n\x15verification_datasets\x18\x0f \x03(\x0b\x32\x44.ProSuite.Microservices.Definitions.QA.QualityVerificationDatasetMsg\"\xcd\x01\n\x1fQualityConditionVerificationMsg\x12\x1c\n\x14quality_condition_id\x18\x01 \x01(\x05\x12\x19\n\x11stop_condition_id\x18\x02 \x01(\x05\x12\x11\n\tfulfilled\x18\x03 \x01(\x08\x12\x13\n\x0b\x65rror_count\x18\x04 \x01(\x05\x12\x14\n\x0c\x65xecute_time\x18\x05 \x01(\x01\x12\x18\n\x10row_execute_time\x18\x06 \x01(\x01\x12\x19\n\x11tile_execute_time\x18\x07 \x01(\x01\"F\n\x1dQualityVerificationDatasetMsg\x12\x12\n\ndataset_id\x18\x01 \x01(\x05\x12\x11\n\tload_time\x18\x02 \x01(\x01\"0\n\x06LogMsg\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x15\n\rmessage_level\x18\x04 \x01(\x05\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'shared_qa_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _WORKCONTEXTMSG._serialized_start=79
  _WORKCONTEXTMSG._serialized_end=297
  _QUALITYSPECIFICATIONMSG._serialized_start=300
  _QUALITYSPECIFICATIONMSG._serialized_end=652
  _XMLQUALITYSPECIFICATIONMSG._serialized_start=654
  _XMLQUALITYSPECIFICATIONMSG._serialized_end=766
  _CONDITIONLISTSPECIFICATIONMSG._serialized_start=769
  _CONDITIONLISTSPECIFICATIONMSG._serialized_end=1000
  _DATASOURCEMSG._serialized_start=1002
  _DATASOURCEMSG._serialized_end=1111
  _QUALITYSPECIFICATIONELEMENTMSG._serialized_start=1114
  _QUALITYSPECIFICATIONELEMENTMSG._serialized_end=1293
  _QUALITYCONDITIONMSG._serialized_start=1296
  _QUALITYCONDITIONMSG._serialized_end=1621
  _INSTANCECONFIGURATIONMSG._serialized_start=1624
  _INSTANCECONFIGURATIONMSG._serialized_end=1817
  _PARAMETERMSG._serialized_start=1820
  _PARAMETERMSG._serialized_end=2025
  _INSTANCEDESCRIPTORMSG._serialized_start=2028
  _INSTANCEDESCRIPTORMSG._serialized_end=2218
  _CLASSDESCRIPTORMSG._serialized_start=2220
  _CLASSDESCRIPTORMSG._serialized_end=2282
  _VERIFICATIONPARAMETERSMSG._serialized_start=2285
  _VERIFICATIONPARAMETERSMSG._serialized_end=3015
  _GDBDATA._serialized_start=3017
  _GDBDATA._serialized_end=3130
  _DATAREQUEST._serialized_start=3133
  _DATAREQUEST._serialized_end=3453
  _CLASSDEF._serialized_start=3455
  _CLASSDEF._serialized_end=3513
  _RELATIONSHIPCLASSQUERY._serialized_start=3516
  _RELATIONSHIPCLASSQUERY._serialized_end=3656
  _SCHEMAMSG._serialized_start=3659
  _SCHEMAMSG._serialized_end=3845
  _SCHEMAREQUEST._serialized_start=3848
  _SCHEMAREQUEST._serialized_end=3983
  _VERIFICATIONPROGRESSMSG._serialized_start=3986
  _VERIFICATIONPROGRESSMSG._serialized_end=4439
  _ISSUEMSG._serialized_start=4442
  _ISSUEMSG._serialized_end=4858
  _INVOLVEDTABLEMSG._serialized_start=4860
  _INVOLVEDTABLEMSG._serialized_end=4918
  _QUALITYVERIFICATIONMSG._serialized_start=4921
  _QUALITYVERIFICATIONMSG._serialized_end=5489
  _QUALITYCONDITIONVERIFICATIONMSG._serialized_start=5492
  _QUALITYCONDITIONVERIFICATIONMSG._serialized_end=5697
  _QUALITYVERIFICATIONDATASETMSG._serialized_start=5699
  _QUALITYVERIFICATIONDATASETMSG._serialized_end=5769
  _LOGMSG._serialized_start=5771
  _LOGMSG._serialized_end=5819
# @@protoc_insertion_point(module_scope)



class NonLinearSegmentType:
    Bezier: tuple = ("BEZIER", 0)
    CircularArc: tuple = ("CIRCULARARC", 1)
    EllipticArc: tuple = ("ELLIPTICARC", 2)

    type_list = [Bezier, CircularArc, EllipticArc]



from typing import Union
from prosuite.perimeter import EnvelopePerimeter, EsriShapePerimeter, WkbPerimeter
from prosuite.verification_parameters import VerificationParameters
from prosuite.specification import Specification
from prosuite.xml_specification import XmlSpecification


class AdvancedParameters(VerificationParameters):
    def __init__(self, specification, output_dir, perimeter, verification_params: VerificationParameters = None) -> None:

        self.specification:  Union[Specification,
                                   XmlSpecification] = specification
        self.perimeter: Union[EnvelopePerimeter,
                              EsriShapePerimeter, WkbPerimeter] = perimeter
        self.output_dir: str = output_dir
        self.tile_size: int = 5000
        self.user_name: str = None
        if (verification_params):
            self.tile_size = verification_params.tile_size
            self.user_name = verification_params.user_name



from prosuite.model import Model


class BaseDataset():
    """
    The base class for datasets representing tabular data. It is either a :class:`.Dataset` or, in the future, a :class:`.Transformer`
    """
    def __init__(self, name: str, filter_expression: str = ""):
        self.name: str = name
        self.filter_expression: str = filter_expression
        pass


class Dataset(BaseDataset):
    """
    A dataset represents data from a table or feature class in a workspace, optionally filtered by an SQL expression.

    :param name: table or featureclass name
    :type name: str
    :param model: The :class:`prosuite.model.Model` this dataset belongs to.
    :type model: class:`prosuite.model.Model`
    :param filter_expression: A where clause that filters the table. The syntax of the where clause is defined in
        the document SQLSyntax_en.pdf
    :type filter_expression: str, optional
    """

    def __init__(self, name: str, model: Model, filter_expression: str = ""):
        super().__init__(name, filter_expression)
        self.model: Model = model
